@@ -42,18 +42,21 @@ class BaseAction(object):
 
         try:
             argspec = inspect.getargspec(func)
-        except TypeError:
+        except TypeError: # a callable object, not a function
             argspec = inspect.getargspec(func.__call__)
         # use task meta information as extra_args
-        extra_args = {
+        meta_args = {
             'task': task,
             'targets': task.targets,
             'dependencies': task.file_dep,
             'changed': task.dep_changed,
             }
 
+        extra_args = dict(meta_args)
         # tasks parameter options
         extra_args.update(task.options)
+        if task.pos_arg is not None:
+            extra_args[task.pos_arg] = task.pos_arg_val
         kwargs = kwargs.copy()
 
         for key in six.iterkeys(extra_args):
@@ -63,10 +66,10 @@ class BaseAction(object):
 
                 # it is forbidden to use default values for this arguments
                 # because the user might be unware of this magic.
-                if (argspec.defaults and
+                if (key in meta_args and argspec.defaults and
                     len(argspec.defaults) > (len(argspec.args) - (arg_pos+1))):
-                    msg = ("%s.%s: '%s' argument default value not allowed "
-                           "(reserved by doit)"
+                    msg = ("Task %s, action %s(): The argument '%s' is not "
+                           "allowed  to have a default value (reserved by doit)"
                            % (task.name, func.__name__, key))
                     raise InvalidTask(msg)
 
@@ -86,14 +89,24 @@ class CmdAction(BaseAction):
     """
     Command line action. Spawns a new process.
 
-    @ivar action(str): Command to be passed to the shell subprocess.
-         It may contain python mapping strings with the keys: dependencies,
+    @ivar action(str,list,callable): subprocess command string or string list,
+         see subprocess.Popen first argument.
+         It may also be a callable that generates the command string.
+         Strings may contain python mappings with the keys: dependencies,
          changed and targets. ie. "zip %(targets)s %(changed)s"
     @ivar task(Task): reference to task that contains this action
     @ivar save_out: (str) name used to save output in `values`
+    @ivar shell: use shell to execute command
+                 see subprocess.Popen `shell` attribute
+    @ivar pkwargs: Popen arguments except 'stdout' and 'stderr'
     """
 
-    def __init__(self, action, task=None, save_out=None): #pylint: disable=W0231
+    def __init__(self, action, task=None, save_out=None, shell=True,
+                 **pkwargs): #pylint: disable=W0231
+        for forbidden in ('stdout', 'stderr'):
+            if forbidden in pkwargs:
+                msg = "CmdAction can't take param named '{0}'."
+                raise InvalidTask(msg.format(forbidden))
         self._action = action
         self.task = task
         self.out = None
@@ -101,12 +114,15 @@ class CmdAction(BaseAction):
         self.result = None
         self.values = {}
         self.save_out = save_out
+        self.shell = shell
+        self.pkwargs = pkwargs
 
     @property
     def action(self):
-        if isinstance(self._action, six.string_types):
+        if isinstance(self._action, (six.string_types, list)):
             return self._action
         else:
+            # action can be a callable that returns a string command
             ref, args, kw = normalize_callable(self._action)
             kwargs = self._prepare_kwargs(self.task, ref, args, kw)
             return ref(*args, **kwargs)
@@ -130,16 +146,14 @@ class CmdAction(BaseAction):
                 process.terminate()
                 input_.read()
                 raise
-            # unbuffered ? process.stdout.read(1)
-            if line:
-                capture.write(line)
-                if realtime:
-                    if sys.version > '3': # pragma: no cover
-                        realtime.write(line)
-                    else:
-                        realtime.write(line.encode(encoding))
-            if not line and process.poll() != None:
+            if not line:
                 break
+            capture.write(line)
+            if realtime:
+                if sys.version > '3': # pragma: no cover
+                    realtime.write(line)
+                else:
+                    realtime.write(line.encode(encoding))
 
 
     def execute(self, out=None, err=None):
@@ -163,8 +177,10 @@ class CmdAction(BaseAction):
             return TaskError("CmdAction Error creating command string", exc)
 
         # spawn task process
-        process = subprocess.Popen(action, shell=True,
-                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        process = subprocess.Popen(
+            action, shell=self.shell,
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            **self.pkwargs)
 
         output = StringIO()
         errput = StringIO()
@@ -208,6 +224,10 @@ class CmdAction(BaseAction):
         if not self.task:
             return self.action
 
+        # cant expand keywords if action is a list of strings
+        if isinstance(self.action, list):
+            return self.action
+
         subs_dict = {'targets' : " ".join(self.task.targets),
                      'dependencies': " ".join(self.task.file_dep)}
         # just included changed if it is set
@@ -215,6 +235,9 @@ class CmdAction(BaseAction):
             subs_dict['changed'] = " ".join(self.task.dep_changed)
         # task option parameters
         subs_dict.update(self.task.options)
+        # convert postional parameters from list space-separated string
+        if self.task.pos_arg:
+            subs_dict[self.task.pos_arg] = ' '.join(self.task.pos_arg_val)
         return self.action % subs_dict
 
     def __str__(self):
@@ -230,7 +253,18 @@ class Writer(object):
     """write to many streams"""
     def __init__(self, *writers):
         """@param writers - file stream like objects"""
-        self.writers = writers
+        self.writers = []
+        self._isatty = True
+        for writer in writers:
+            self.add_writer(writer)
+
+    def add_writer(self, stream, isatty=None):
+        """adds a stream to the list of writers
+        @param isatty: (bool) if specified overwrites real isatty from stream
+        """
+        self.writers.append(stream)
+        isatty = stream.isatty() if (isatty is None) else isatty
+        self._isatty = self._isatty and isatty
 
     def write(self, text):
         """write 'text' to all streams"""
@@ -242,6 +276,9 @@ class Writer(object):
         for stream in self.writers:
             stream.flush()
 
+    def isatty(self):
+        return self._isatty
+
 
 class PythonAction(BaseAction):
     """Python action. Execute a python callable.
@@ -306,18 +343,20 @@ class PythonAction(BaseAction):
         # set std stream
         old_stdout = sys.stdout
         output = StringIO()
+        out_writer = Writer()
+        # capture output but preserve isatty() from original stream
+        out_writer.add_writer(output, old_stdout.isatty())
+        if out:
+            out_writer.add_writer(out)
+        sys.stdout = out_writer
+
         old_stderr = sys.stderr
         errput = StringIO()
-
-        out_list = [output]
-        if out:
-            out_list.append(out)
-        err_list = [errput]
+        err_writer = Writer()
+        err_writer.add_writer(errput, old_stderr.isatty())
         if err:
-            err_list.append(err)
-
-        sys.stdout = Writer(*out_list)
-        sys.stderr = Writer(*err_list)
+            err_writer.add_writer(err)
+        sys.stderr = err_writer
 
         kwargs = self._prepare_kwargs()
 
@@ -373,7 +412,10 @@ def create_action(action, task_ref):
         return action
 
     if isinstance(action, six.string_types):
-        return CmdAction(action, task_ref)
+        return CmdAction(action, task_ref, shell=True)
+
+    if isinstance(action, list):
+        return CmdAction(action, task_ref, shell=False)
 
     if isinstance(action, tuple):
         if len(action) > 3:
