@@ -1,10 +1,29 @@
 import inspect
 import sys
+from collections import deque
 
+from . import version
 from .cmdparse import CmdOption, CmdParse
+from .exceptions import InvalidCommand, InvalidDodoFile
+from .dependency import backend_map
 from . import loader
 
 
+def version_tuple(ver_in):
+    """convert a version string or tuple into a 3-element tuple with ints
+    Any part that is not a number (dev0, a2, b4) will be converted to -1
+    """
+    result = []
+    parts = ver_in.split('.') if isinstance(ver_in, str) else ver_in
+    for rev in parts:
+        try:
+            result.append(int(rev))
+        except:
+            result.append(-1)
+    assert len(result) == 3
+    return result
+
+
 class Command(object):
     """third-party should subclass this for commands that do no use tasks
 
@@ -32,6 +51,7 @@ class Command(object):
         self.name = self.name or self.__class__.__name__.lower()
         Command.CMD_LIST.append(self.name)
         self.options = self.set_options()
+        self.opt_values = None # option values
 
     def set_options(self):
         """@reutrn list of CmdOption
@@ -55,6 +75,7 @@ class Command(object):
         @returns: result of self.execute
         """
         params, args = CmdParse(self.options).parse(in_args)
+        self.opt_values = params
         return self.execute(params, args)
 
 
@@ -87,6 +108,17 @@ opt_depfile = {'name': 'dep_file',
                'help': "file used to save successful runs"
                }
 
+# dependency file DB backend
+opt_backend = {
+    'name': 'backend',
+    'short':'',
+    'long': 'backend',
+    'type': str,
+    'default': "dbm",
+    'help': ("Select dependency file backend." +
+             "Available options dbm, json, sqlite3. [default: %(default)s]")
+}
+
 
 #### options related to dodo.py
 # select dodo file containing tasks
@@ -110,7 +142,7 @@ opt_cwd = {'name': 'cwdPath',
 
 # seek dodo file on parent folders
 opt_seek_file = {'name': 'seek_file',
-                 'short': '',
+                 'short': 'k',
                  'long': 'seek-file',
                  'type': bool,
                  'default': False,
@@ -181,13 +213,16 @@ class DoitCmdBase(Command):
     cmd_options => list of option dictionary (see CmdOption)
     _execute => method, argument names must be option names
     """
-    base_options = (opt_depfile,)
+    base_options = (opt_depfile, opt_backend)
 
-    def __init__(self, task_loader=None, dep_file=None, config=None,
-                 task_list=None, sel_tasks=None, outstream=None):
+    def __init__(self, task_loader=None, dep_file=None, backend=None,
+                 config=None, task_list=None, sel_tasks=None, outstream=None):
         """this initializer is usually just used on tests"""
+        # FIXME 'or TaskLoader()' below is hack for tests
         self._loader = task_loader or TaskLoader()
         Command.__init__(self)
+        # TODO: helper to test code should not be here
+        self.dep_class = backend_map.get(backend)
         self.dep_file = dep_file   # (str) filename usually '.doit.db'
         self.config = config or {} # config from dodo.py & cmdline
         self.task_list = task_list # list of tasks
@@ -209,13 +244,24 @@ class DoitCmdBase(Command):
         """load dodo.py, set attributes and call self._execute"""
         self.task_list, self.config = self._loader.load_tasks(self, params,
                                                               args)
+        self.sel_tasks = args or self.config.get('default_tasks')
+
+        # check minversion
+        minversion = self.config.get('minversion')
+        if minversion:
+            if version_tuple(minversion) > version_tuple(version.VERSION):
+                msg = ('Please update doit. '
+                'Minimum version required is {required}. '
+                'You are using {actual}. ')
+                raise InvalidDodoFile(msg.format(required=minversion,
+                                                 actual=version.VERSION))
 
         # merge config values into params
         params.update_defaults(self.config)
         self.dep_file = params['dep_file']
+        self.dep_class = backend_map.get(params['backend'])
         params['pos_args'] = args # hack
         params['continue_'] = params.get('continue') # hack
-        self.sel_tasks = args or self.config.get('default_tasks')
 
         # magic - create dict based on signature of _execute method
         args_name = inspect.getargspec(self._execute)[0]
@@ -223,4 +269,46 @@ class DoitCmdBase(Command):
         return self._execute(**exec_params)
 
 
+# helper functions to find list of tasks
 
+
+def check_tasks_exist(tasks, name_list):
+    """check task exist"""
+    if not name_list:
+        return
+    for task_name in name_list:
+        if task_name not in tasks:
+            msg = "'%s' is not a task."
+            raise InvalidCommand(msg % task_name)
+
+
+# this is used by commands that do not execute tasks (list, clean, forget...)
+def tasks_and_deps_iter(tasks, sel_tasks, yield_duplicates=False):
+    """iterator of select_tasks and its dependencies
+    @param tasks (dict - Task)
+    @param sel_tasks(list - str)
+    """
+    processed = set() # str - task name
+    to_process = deque(sel_tasks) # str - task name
+    # get initial task
+    while to_process:
+        task = tasks[to_process.popleft()]
+        processed.add(task.name)
+        yield task
+        # FIXME this does not take calc_dep into account
+        for task_dep in task.task_dep + task.setup_tasks:
+            if (task_dep not in processed) and (task_dep not in to_process):
+                to_process.append(task_dep)
+            elif yield_duplicates:
+                yield tasks[task_dep]
+
+
+def subtasks_iter(tasks, task):
+    """find all subtasks for a given task
+    @param tasks (dict - Task)
+    @param task (Task)
+    """
+    for name in task.task_dep:
+        dep = tasks[name]
+        if dep.is_subtask:
+            yield dep
