@@ -2,27 +2,32 @@ from .cmd_base import DoitCmdBase
 from .cmd_base import check_tasks_exist, tasks_and_deps_iter, subtasks_iter
 
 
-opt_clean_dryrun = {'name': 'dryrun',
-                    'short': 'n', # like make dry-run
-                    'long': 'dry-run',
-                    'type': bool,
-                    'default': False,
-                    'help': 'print actions without really executing them'}
+opt_clean_dryrun = {
+    'name': 'dryrun',
+    'short': 'n', # like make dry-run
+    'long': 'dry-run',
+    'type': bool,
+    'default': False,
+    'help': 'print actions without really executing them',
+    }
 
-opt_clean_cleandep = {'name': 'cleandep',
-                    'short': 'c', # clean
-                    'long': 'clean-dep',
-                    'type': bool,
-                    'default': False,
-                    'help': 'clean task dependencies too'}
+opt_clean_cleandep = {
+    'name': 'cleandep',
+    'short': 'c', # clean
+    'long': 'clean-dep',
+    'type': bool,
+    'default': False,
+    'help': 'clean task dependencies too',
+    }
 
 opt_clean_cleanall = {
     'name': 'cleanall',
-    'short': 'a', # clean
+    'short': 'a', # all
     'long': 'clean-all',
     'type': bool,
     'default': False,
-    'help': 'clean all task'}
+    'help': 'clean all task',
+    }
 
 
 class Clean(DoitCmdBase):
@@ -76,13 +81,12 @@ class Clean(DoitCmdBase):
             # including repeated entries will garantee that deps are listed
             # first when the list is reversed
             to_clean = list(tasks_and_deps_iter(tasks, clean_list, True))
-            to_clean.reverse()
         # include only subtasks in list
         else:
             to_clean = []
-            for name in clean_list:
+            for name in reversed(clean_list):
                 task = tasks[name]
-                to_clean.extend(subtasks_iter(tasks, task))
                 to_clean.append(task)
-
+                to_clean.extend(subtasks_iter(tasks, task))
+        to_clean.reverse()
         self.clean_tasks(to_clean, dryrun)
