@@ -1,6 +1,7 @@
 """Control tasks execution order"""
 import fnmatch
 from collections import deque
+import six
 
 from .exceptions import InvalidTask, InvalidCommand, InvalidDodoFile
 from .task import Task
@@ -51,7 +52,7 @@ class TaskControl(object):
             self._def_order.append(task.name)
 
         # expand wild-card task-dependencies
-        for task in self.tasks.itervalues():
+        for task in six.itervalues(self.tasks):
             for pattern in task.wild_dep:
                 task.task_dep.extend(self._get_wild_tasks(pattern))
 
@@ -62,7 +63,7 @@ class TaskControl(object):
     def _check_dep_names(self):
         """check if user input task_dep or setup_task that doesnt exist"""
         # check task-dependencies exist.
-        for task in self.tasks.itervalues():
+        for task in six.itervalues(self.tasks):
             for dep in task.task_dep:
                 if dep not in self.tasks:
                     msg = "%s. Task dependency '%s' does not exist."
@@ -78,7 +79,7 @@ class TaskControl(object):
         """get task_dep based on file_dep on a target from another task"""
         # 1) create a dictionary associating every target->task. where the task
         # builds that target.
-        for task in self.tasks.itervalues():
+        for task in six.itervalues(self.tasks):
             for target in task.targets:
                 if target in self.targets:
                     msg = ("Two different tasks can't have a common target." +
@@ -88,7 +89,7 @@ class TaskControl(object):
                 self.targets[target] = task.name
         # 2) now go through all dependencies and check if they are target from
         # another task.
-        for task in self.tasks.itervalues():
+        for task in six.itervalues(self.tasks):
             self.add_implicit_task_dep(self.targets, task, task.file_dep)
 
 
@@ -112,16 +113,26 @@ class TaskControl(object):
     def _process_filter(self, task_selection):
         """process cmd line task options
         [task_name [-task_opt [opt_value]] ...] ...
+
+        @param task_selection: list of strings with task names/params or target
+        @return list of task names. Expanding glob and removed params
         """
         filter_list = []
         def add_filtered_task(seq, f_name):
-            """can be filter by target or task name """
+            """add task to list `filter_list` and set task.options from params
+            @return list - str: of elements not yet
+            """
             filter_list.append(f_name)
+            # only tasks specified by name can contain parameters
             if f_name in self.tasks:
                 # parse task_selection
                 the_task = self.tasks[f_name]
                 # remaining items are other tasks not positional options
                 the_task.options, seq = the_task.taskcmd.parse(seq)
+                # if task takes positional parameters set all as pos_arg_val
+                if the_task.pos_arg is not None:
+                    the_task.pos_arg_val = seq
+                    seq = []
             return seq
 
         # process...
@@ -141,7 +152,7 @@ class TaskControl(object):
     def _filter_tasks(self, task_selection):
         """Select tasks specified by filter.
 
-        filter can specify tasks to be execute by task name or target.
+        @param task_selection: list of strings with task names/params or target
         @return (list) of string. where elements are task name.
         """
         selected_task = []
@@ -163,7 +174,10 @@ class TaskControl(object):
 
 
     def process(self, task_selection):
-        """@return (list - string) each element is the name of a task"""
+        """
+        @param task_selection: list of strings with task names/params
+        @return (list - string) each element is the name of a task
+        """
         # execute only tasks in the filter in the order specified by filter
         if task_selection is not None:
             self.selected_tasks = self._filter_tasks(task_selection)
@@ -173,14 +187,13 @@ class TaskControl(object):
             self.selected_tasks = self._def_order
 
 
-    def task_dispatcher(self, include_setup=False):
+    def task_dispatcher(self):
         """return a TaskDispatcher generator
         """
         assert self.selected_tasks is not None, \
             "must call 'process' before this"
 
-        return TaskDispatcher(self.tasks, self.targets, self.selected_tasks,
-                              include_setup)
+        return TaskDispatcher(self.tasks, self.targets, self.selected_tasks)
 
 
 
@@ -261,14 +274,10 @@ class TaskDispatcher(object):
     """Dispatch another task to be selected/executed, mostly handle with MP
 
     Note that a dispatched task might not be ready to be executed.
-
-    @ivar include_setup: (bool) when True tasks wont be execute so
-                         do not wait for task deps.
     """
-    def __init__(self, tasks, targets, selected_tasks, include_setup=False):
+    def __init__(self, tasks, targets, selected_tasks):
         self.tasks = tasks
         self.targets = targets
-        self.include_setup = include_setup
 
         self.nodes = {} # key task-name, value: ExecNode
         # queues
@@ -310,6 +319,9 @@ class TaskDispatcher(object):
                 wait_for.add(name)
             else:
                 node.parent_status(dep_node)
+                if calc:
+                    self._process_calc_dep_results(dep_node, node)
+
 
         # update ExecNode setting parent/dependent relationship
         for name in wait_for:
@@ -338,18 +350,23 @@ class TaskDispatcher(object):
         # add calc_dep & task_dep until all processed
         # calc_dep may add more deps so need to loop until nothing left
         while True:
-            for calc_dep in node.calc_dep:
-                yield self._gen_node(node, calc_dep)
-            self._node_add_wait_run(node, node.calc_dep, calc=True)
+            calc_dep_list = list(node.calc_dep)
             node.calc_dep.clear()
-
-            # add task_dep
-            for task_dep in node.task_dep:
-                yield self._gen_node(node, task_dep)
-            self._node_add_wait_run(node, node.task_dep)
+            task_dep_list = node.task_dep[:]
             node.task_dep = []
 
-            if (node.wait_run or node.wait_run_calc) and not self.include_setup:
+            for calc_dep in calc_dep_list:
+                yield self._gen_node(node, calc_dep)
+            self._node_add_wait_run(node, calc_dep_list, calc=True)
+
+            # add task_dep
+            for task_dep in task_dep_list:
+                yield self._gen_node(node, task_dep)
+            self._node_add_wait_run(node, task_dep_list)
+
+            if (node.calc_dep or node.task_dep):
+                continue # pragma: no cover # coverage cant catch this #198
+            elif (node.wait_run or node.wait_run_calc):
                 yield 'wait'
             else:
                 break
@@ -367,7 +384,7 @@ class TaskDispatcher(object):
                 yield "wait"
 
             # if this task should run, so schedule setup-tasks before itself
-            if node.run_status == 'run' or self.include_setup:
+            if node.run_status == 'run':
                 for setup_task in this_task.setup_tasks:
                     yield self._gen_node(node, setup_task)
                 self._node_add_wait_run(node, this_task.setup_tasks)
@@ -431,21 +448,8 @@ class TaskDispatcher(object):
                 # calc_dep might add new deps that can be run without
                 # waiting for the completion of the remaining deps
                 is_ready = True
+                self._process_calc_dep_results(node, waiting_node)
 
-                # refresh this task dependencies with values got from calc_dep
-                values = node.task.values
-                len_task_deps = len(node.task.task_dep)
-                old_calc_dep = node.task.calc_dep.copy()
-                waiting_node.task.update_deps(values)
-                TaskControl.add_implicit_task_dep(
-                    self.targets, waiting_node.task,
-                    values.get('file_dep', []))
-
-                # update node's list of non-processed dependencies
-                new_task_dep = waiting_node.task.task_dep[len_task_deps:]
-                waiting_node.task_dep.extend(new_task_dep)
-                new_calc_dep = waiting_node.task.calc_dep - old_calc_dep
-                waiting_node.calc_dep.update(new_calc_dep)
 
             # this node can be further processed
             if is_ready and (waiting_node in self.waiting):
@@ -453,6 +457,24 @@ class TaskDispatcher(object):
                 self.waiting.remove(waiting_node)
 
 
+    def _process_calc_dep_results(self, node, waiting_node):
+        # refresh this task dependencies with values got from calc_dep
+        values = node.task.values
+        len_task_deps = len(waiting_node.task.task_dep)
+        old_calc_dep = waiting_node.task.calc_dep.copy()
+        waiting_node.task.update_deps(values)
+        TaskControl.add_implicit_task_dep(
+            self.targets, waiting_node.task,
+            values.get('file_dep', []))
+
+        # update node's list of non-processed dependencies
+        new_task_dep = waiting_node.task.task_dep[len_task_deps:]
+        waiting_node.task_dep.extend(new_task_dep)
+        new_calc_dep = waiting_node.task.calc_dep - old_calc_dep
+        waiting_node.calc_dep.update(new_calc_dep)
+
+
+
     def _dispatcher_generator(self, selected_tasks):
         """return generator dispatching tasks"""
         # each selected task will create a tree (from dependencies) of
@@ -493,8 +515,5 @@ class TaskDispatcher(object):
             # got 'wait', add ExecNode to waiting queue
             else:
                 assert next_step == "wait"
-                # skip all waiting tasks, just getting a list of tasks...
-                if not self.include_setup:
-                    self.waiting.add(node)
-                    node = None
-
+                self.waiting.add(node)
+                node = None
