@@ -2,6 +2,8 @@
 
 import os
 import hashlib
+import subprocess
+import inspect
 import six
 if six.PY3: # pragma: no cover
     from dbm import dumb
@@ -360,11 +362,11 @@ class SqliteDB(object):
 class DependencyBase(object):
     """Manage tasks dependencies (abstract class)
 
-    Each dependency is a saved in "db". the "db" can have json or dbm
+    Each dependency is saved in "db". the "db" can have json or dbm
     format where there is a dictionary for every task. each task has a
     dictionary where key is a dependency (abs file path), and the value is the
     dependency signature.
-    Apart from dependencies onther values are also saved on the task dictionary
+    Apart from dependencies other values are also saved on the task dictionary
      * 'result:', 'task:<task-name>', 'ignore:'
      * user(task) defined values are defined in '_values_:' sub-dict
 
@@ -414,6 +416,10 @@ class DependencyBase(object):
             size = os.path.getsize(dep)
             self._set(task.name, dep, (timestamp, size, get_file_md5(dep)))
 
+        # save list of file_deps
+        self._set(task.name, 'deps:', tuple(task.file_dep))
+
+
 
     def get_values(self, task_name):
         """get all saved values from a task
@@ -469,13 +475,32 @@ class DependencyBase(object):
             # if parameter is a callable
             if hasattr(utd, '__call__'):
                 # FIXME control verbosity, check error messages
-                # 1) prepare arguments
-                args = [task, self.get_values(task.name)] + utd_args
-                # 2) setup object with global info all tasks
+                # 1) setup object with global info all tasks
                 if isinstance(utd, UptodateCalculator):
                     utd.setup(self, tasks_dict)
+                # 2) add magic positional args for `task` and `values`
+                # if present.
+                # get args removing self if present
+                if inspect.isfunction(utd):
+                    spec_args = inspect.getargspec(utd).args
+                elif inspect.ismethod(utd):
+                    spec_args = inspect.getargspec(utd).args[1:]
+                else:
+                    spec_args = inspect.getargspec(utd.__call__).args[1:]
+                magic_args = []
+                for i, name in enumerate(spec_args):
+                    if i == 0 and name == 'task':
+                        magic_args.append(task)
+                    elif i == 1 and name == 'values':
+                        magic_args.append(self.get_values(task.name))
+                args = magic_args + utd_args
                 # 3) call it and get result
                 uptodate_result = utd(*args, **utd_kwargs)
+            elif isinstance(utd, six.string_types):
+                # TODO py3.3 has subprocess.DEVNULL
+                with open(os.devnull, 'wb') as null:
+                    uptodate_result = subprocess.call(
+                        utd, shell=True, stderr=null, stdout=null) == 0
             # parameter is a value
             else:
                 uptodate_result = utd
@@ -500,8 +525,12 @@ class DependencyBase(object):
                 return 'run'
 
         # check for modified file_dep
-        changed = []
-        status = 'up-to-date' # initial assumption
+        changed = [] # list of file_dep that changed
+        previous = self._get(task.name, 'deps:')
+        if previous and set(previous) != task.file_dep:
+             status = 'run'
+        else:
+            status = 'up-to-date' # initial assumption
         for dep in tuple(task.file_dep):
             try:
                 file_stat = os.stat(dep)
