@@ -12,6 +12,16 @@ from .exceptions import CatchedException, InvalidTask
 from .action import create_action, PythonAction
 
 
+def first_line(doc):
+    """extract first non-blank line from text, to extract docstring title"""
+    if doc is not None:
+        for line in doc.splitlines():
+            striped = line.strip()
+            if striped:
+                return striped
+    return ''
+
+
 class Task(object):
     """Task
 
@@ -41,6 +51,9 @@ class Task(object):
 
     @ivar options: (dict) calculated params values (from getargs and taskopt)
     @ivar taskopt: (cmdparse.CmdParse)
+    @ivar pos_arg: (str) name of parameter in action to receive positional
+                     parameters from command line
+    @ivar pos_arg_val: (list - str) list of positional parameters values
     @ivar custom_title: function reference that takes a task object as
                         parameter and returns a string.
     """
@@ -61,9 +74,11 @@ class Task(object):
                   'teardown': ((list, tuple), ()),
                   'doc': (string_types, (None,)),
                   'params': ((list, tuple,), ()),
+                  'pos_arg': (string_types, (None,)),
                   'verbosity': ((), (None,0,1,2,)),
                   'getargs': ((dict,), ()),
                   'title': ((types.FunctionType,), (None,)),
+                  'watch': ((list, tuple), ()),
                   }
 
 
@@ -71,7 +86,9 @@ class Task(object):
                  task_dep=(), uptodate=(),
                  calc_dep=(), setup=(), clean=(), teardown=(),
                  is_subtask=False, has_subtask=False,
-                 doc=None, params=(), verbosity=None, title=None, getargs=None):
+                 doc=None, params=(), pos_arg=None,
+                 verbosity=None, title=None, getargs=None,
+                 watch=()):
         """sanity checks and initialization
 
         @param params: (list of dict for parameters) see cmdparse.CmdOption
@@ -90,14 +107,20 @@ class Task(object):
         self.check_attr(name, 'teardown', teardown, self.valid_attr['teardown'])
         self.check_attr(name, 'doc', doc, self.valid_attr['doc'])
         self.check_attr(name, 'params', params, self.valid_attr['params'])
+        self.check_attr(name, 'pos_arg', pos_arg,
+                        self.valid_attr['pos_arg'])
         self.check_attr(name, 'verbosity', verbosity,
                         self.valid_attr['verbosity'])
         self.check_attr(name, 'getargs', getargs, self.valid_attr['getargs'])
         self.check_attr(name, 'title', title, self.valid_attr['title'])
+        self.check_attr(name, 'watch', watch, self.valid_attr['watch'])
 
         self.name = name
         self.taskcmd = TaskParse([CmdOption(opt) for opt in params])
-        self.options = None
+        self.params = params # save just for use on command `info`
+        self.options = self._init_options()
+        self.pos_arg = pos_arg
+        self.pos_arg_val = None # to be set when parsing command line
         self.setup_tasks = list(setup)
 
         # actions
@@ -133,6 +156,7 @@ class Task(object):
 
         self.teardown = [create_action(a, self) for a in teardown]
         self.doc = self._init_doc(doc)
+        self.watch = watch
 
 
     def _init_deps(self, file_dep, task_dep, calc_dep):
@@ -173,9 +197,11 @@ class Task(object):
                 args = list(item[1]) if len(item)>1 else []
                 kwargs = item[2] if len(item)>2 else {}
                 uptodate.append((call, args, kwargs))
+            elif isinstance(item, six.string_types):
+                uptodate.append((item, [], {}))
             else:
                 msg = ("%s. task invalid 'uptodate' item '%r'. " +
-                       "Must be bool, None, callable or tuple " +
+                       "Must be bool, None, str, callable or tuple " +
                        "(callable, args, kwargs).")
                 raise InvalidTask(msg % (self.name, item))
         return uptodate
@@ -229,16 +255,12 @@ class Task(object):
         """put default values on options. this will be overwritten, if params
         options were passed on the command line.
         """
-        # FIXME call _init_options just once and not check if this.options is None
-        # because its value may be set in a different place... (like uptodate())
-        if self.options is None:
-            # ignore positional parameters
-            self.options = self.taskcmd.parse('')[0]
+        # ignore positional parameters
+        return self.taskcmd.parse('')[0]
 
 
     def _init_getargs(self):
         """task getargs attribute define implicit task dependencies"""
-        self._init_options()
         for arg_name, desc in six.iteritems(self.getargs):
 
             # tuple (task_id, key_name)
@@ -258,13 +280,7 @@ class Task(object):
     def _init_doc(doc):
         """process task "doc" attribute"""
         # store just first non-empty line as documentation string
-        if doc is not None:
-            for line in doc.splitlines():
-                striped = line.strip()
-                if striped:
-                    return striped
-        return ''
-
+        return first_line(doc)
 
     @staticmethod
     def check_attr(task, attr, value, valid):
@@ -292,7 +308,6 @@ class Task(object):
     @property
     def actions(self):
         """lazy creation of action instances"""
-        self._init_options()
         if self._action_instances is None:
             self._action_instances = [create_action(a, self) for a in self._actions]
         return self._action_instances
@@ -334,7 +349,6 @@ class Task(object):
         """Executes task's teardown
         @return failure: see CmdAction.execute
         """
-        self._init_options()
         task_stdout, task_stderr = self._get_out_err(out, err, verbosity)
         for action in self.teardown:
             action_return = action.execute(task_stdout, task_stderr)
@@ -348,7 +362,6 @@ class Task(object):
         @ivar dryrun (bool): if True clean tasks are not executed
                              (just print out what would be executed)
         """
-        self._init_options()
         # if clean is True remove all targets
         if self._remove_targets is True:
             clean_targets(self, dryrun)
